#!/usr/bin/perl -w

############################################################
# short hack to sort city names for Freeciv rulesets.
# based on formula suggested by Mika Korhonen on freeciv-dev.

# Copyleft 1999 by Jarda Benkovsky <pvt.benkovsk@pvtnet.cz>
# Distributed under GNU General Public License (GPL)


############################################################

use strict;
use vars qw{ $line_no %city_list };

# default used if city founding year is not specified. In Czech,
# most cities were founded until 15th century.

my $DEF_YEAR = 1400;

# default population used if not specified in a file

my $DEF_POP  = 1000;

# max line width used for output
my $LINE_WIDTH = 60;

# some koeficients to modify original formula. To emphasize
# city age, make koef_age > 1 and koef_pop < 1 and vice versa.
my $koef_age = 2;
my $koef_pop = 0.5;
my $age_div  = 1000;

############################################################
# compute city weight

sub city_weight {
  my ($city) = @_;

  return $koef_pop * log ($city->{'population'}) + $koef_age * log ((2000 - $city->{'year'}) / $age_div);
}

############################################################
# compare cities by their weights

sub sort_fn {
  my ($a, $b) = @_;

  return &city_weight ($b) <=> &city_weight ($a);
}

############################################################

if (! defined ($ARGV[0])) {
  die "Syntax: sort_cities filename\n";
}

# read the input file

open (FILE, $ARGV[0]) || die "Can't open `$ARGV[0]'\n";

$line_no = 0;
while (defined (my $line = <FILE>)) {
  $line_no++;
  chomp ($line);

  $line =~ s/^\s*\"?//o;
  $line =~ s/\"?\,?\s*$//o;

  next  if ($line eq '' || $line =~ /^\#/o);

  my ($city, undef, $year, $population) = split (/:/, $line);

  if (!defined ($city_list{$city})) {
    if (defined ($year)) { $year =~ tr/x/0/; }
    else { $year = $DEF_YEAR; }

    if (!$population) {
      $population = $DEF_POP;
    }

    $city_list{$city} = {
			 'name' => $city,
			 'year' => $year,
			 'population' => $population
			};
  }
  else {
    # here it is assumed that the second definition is worthless
    print STDERR "`$city' already used (line $line_no)\n";
  }
}

close (FILE);


# print the sorted results

my $line_len = 0;

foreach my $city (sort { &sort_fn ($a, $b) } values %city_list) {

  # print city names along with their computed values
  #  print $city->{'name'} . " : " . sprintf ("%1.4f", &city_weight ($city)) . "\n";

  # print city name in format suitable for Freeciv rulesets
  my $pname =  '"' . $city->{'name'} . '", ';
  $line_len += length ($pname);

  if ($line_len > $LINE_WIDTH) {
    print "\n";
    $line_len = length ($pname);
  }

  print $pname;
}

print "\n";


############################################################
# End of file sort_cities.pl
